// ambiente.h

// ESTRUCTURAS DE DATOS PARA DEFIR EL AMBIENTE DE INTERACCION

// Jess Manuel Olivares Ceja
#define MAX_PALABRA 30
#define SI  1
#define NO  0
#define NO_HAY  -1

//---------------------------------------------------------------------------
// VARIABLES GLOBALES
//---------------------------------------------------------------------------
typedef struct EstructuraGlobal
{
    char nombre[MAX_PALABRA] ;
    char tipo ;
    int dimension1;
    int dimension2;
    union
    {
        long entero;
        double flotante;
        char byte;
        char *cadena;
        long *enteros;
        double *flotantes;
        struct EstructuraMensaje *puerto ;
    } valor ;
    long tiempo;           // TIEMPO EN QUE ESTARA DISPONIBLE
    struct EstructuraGlobal *sig ;
} TipoGlobal;

//---------------------------------------------------------------------------
// VARIABLES REGIONALES
//---------------------------------------------------------------------------
typedef struct EstructuraRegional
{
    char nombre[MAX_PALABRA] ;
    char tipo ;
    int dimension1;
    int dimension2;
    union
    {
        long entero;
        double flotante;
        char byte;
        char *cadena;
        long *enteros;
        double *flotantes;
        //variables tipo puerto
    } valor ;
    long tiempo;           // TIEMPO EN QUE ESTARA DISPONIBLE
    struct EstructuraRegional *sig ;
} TipoRegional;

//---------------------------------------------------------------------------
// AGENTES
//---------------------------------------------------------------------------
typedef struct EstructuraProposito
{
    char proposito[MAX_PALABRA] ;
    int estado; // 0=sin cumplir, 1=cumplido
    struct EstructuraProposito *sig ;
} TipoProposito ;

typedef struct EstructuraInterna
{
    char nombre[MAX_PALABRA] ;
    char tipo ;
    int dimension1;
    int dimension2;
    union
    {
        long entero;
        double flotante;
        char byte;
        char *cadena;
        long *enteros;
        double *flotantes;
    } valor ;
    struct EstructuraInterna *sig ;
} TipoInterna;

typedef struct EstructuraVRegion // variables regionales que ve el agente
{
    char nombre[MAX_PALABRA] ;
    struct EstructuraVRegion *sig ;
} TipoVRegion ;

typedef struct EstructuraEvento // Evento Inesperado que percibe
{
    char evento[MAX_PALABRA] ;
    struct EstructuraEvento *sig ;
} TipoEvento ;

typedef struct EstructuraAccion
{
    char accion[MAX_PALABRA] ;
    struct EstructuraAccion *sig ;
} TipoAccion ;

typedef struct EstructuraInicial
{
    char inicial[MAX_PALABRA] ;
    struct EstructuraInicial *sig ;
} TipoInicial ;

typedef struct EstructuraIdHebra
{
    struct EstructuraHebra *hebra ;
    struct EstructuraIdHebra *sig ;
} TipoIdHebra ;

typedef struct EstructuraAgente
{
    long id ;  //  IDENTIFICACION UNICA DE CADA AGENTE
    char nombre[MAX_PALABRA] ; // NOMBRE DEL AGENTE
    TipoProposito *proposito ;  // LISTA DE PROPOSITOS
    TipoInterna *interna ; // LISTA DE VARIABLES INTERNAS
    TipoVRegion *regional ; // LISTA DE VARIABLES REGIONALES
    TipoEvento *evento ; //
    TipoAccion *accion ; //
    TipoInicial *inicial ; // PAPELES CON LOS QUE INICIA EL AGENTE
    TipoIdHebra *hebra ;  // LISTA DE HEBRAS DEL AGENTE
    struct EstructuraNodoPlan *plan ; // PLAN QUE DESARROLLA EL AGENTE
    struct EstructuraAgente *agente ; // AGENTE PADRE DE LA INSTANCIA
    struct EstructuraAgente *instancia ; // LISTA DE INSTANCIAS DE CADA AGENTE
    struct EstructuraAgente *sig ;
} TipoAgente;

//---------------------------------------------------------------------------
// ESPACIOS DE INTERACCION
//---------------------------------------------------------------------------
typedef struct EstructuraLocal
{
    char nombre[MAX_PALABRA] ;
    char tipo ;
    int dimension1;
    int dimension2;
    union
    {
        long entero;
        double flotante;
        char byte;
        char *cadena;
        long *enteros;
        double *flotantes;
    } valor ;
    struct EstructuraLocal *sig ;
} TipoLocal;

typedef struct EstructuraRequisito
{
    char requisito[MAX_PALABRA] ;
    struct EstructuraRequisito *sig ;
} TipoRequisito ;

typedef struct EstructuraInstruccion
{
    char opcode[8];
    char op1[MAX_PALABRA];
    char op2[MAX_PALABRA];
    char op3[MAX_PALABRA];
} TipoInstruccion;

typedef struct EstructuraPapel
{
    char nombre[MAX_PALABRA] ;  // NOMBRE DEL PAPEL
    int cupo ;  // CUPO DEL MAXIMO DE INSTANCIAS DE ESTE PAPEL
    int usados ; //
    TipoRequisito *requisito ;
    TipoLocal *local ;
    int u_instruccion ;
    TipoInstruccion *instruccion ;
    struct EstructuraPapel *sig ;
} TipoPapel ;

typedef struct EstructuraInteraccion
{
    char nombre[MAX_PALABRA] ; // NOMBRE DE LA INTERACCION
    TipoPapel *papel ;
    struct EstructuraInteraccion *origen ;
    struct EstructuraInteraccion *instancia ;
    struct EstructuraInteraccion *sig ;
} TipoInteraccion ;

//---------------------------------------------------------------------------
// MAIN
//---------------------------------------------------------------------------
typedef struct EstructuraHebra TipoHebra ;

typedef struct EstructuraMain
{
    TipoPapel *papel ;
    TipoHebra *hebra ;
} TipoMain ;

//---------------------------------------------------------------------------
// AMBIENTE DE EJECUCION
//---------------------------------------------------------------------------
typedef struct EstructuraCola
{
    TipoHebra *hebra ;
    struct EstructuraCola *sig ;
} TipoCola ;

typedef struct EstructuraStack
{
    double valor ;
    struct EstructuraStack *sig ;
} TipoStack ;

typedef struct EstructuraHebra
{
    long id ;  // IDENTIFICACION UNICA DEL PAPEL EN EJECUCION
    double R ,tA ,tB ; // REGISTROS INTERNOS DE CADA PAPEL ACTIVO
    int condicion ;
    long tiempo ; // TIEMPO DE LA SIGUIENTE INSTRUCCION A EJECUTARSE, -1=ESPERA
    int PC ; // PROGRAM COUNTER, INDICA LA INSTRUCCION EN QUE SE ENCUENTRA
    int estado ; // 0=ejecutandose, 1=suspendido, 2=terminado
    TipoPapel *papel ; // APUNTA AL PAPEL QUE CONTIENE SUS INSTRUCCIONES
    TipoProposito *proposito ; // LISTA PROPOSITOS QUE MANTIENEN ACTIVO AL PAPEL
    TipoStack *stack ;  // STACK DE EJECUCION
    TipoLocal *local ; // LISTA DE VARIABLES LOCALES
    struct EstructuraAgente *agente ; // APUNTA A SU AGENTE PROPIETARIO
    struct EstructuraHebra *sig ;
} TipoHebra ;
//---------------------------------------------------------------------------
// PLANIFICADOR
//---------------------------------------------------------------------------
typedef struct EstructuraNodoPlan  // NODO CON UN PAPEL Y SUS REQUISITOS
{                              // UTILIZADO EN LA CARGA DE LOS PLANES POSIBLES
    char papel[MAX_PALABRA];
    TipoRequisito *requisito ;
    TipoProposito *proposito ;
    struct EstructuraNodoPlan *sig ;
} TipoNodoPlan ;

typedef struct EstructuraMultiPlan  // NODO PARA APUNTAR A LOS PAPELES POSIBLES
{                    // QUE PUEDE UTILIZAR UN AGENTE
    TipoNodoPlan *posible ;
    int beneficios ;
    int necesita ;
    struct EstructuraMultiPlan *sig ;
} TipoMultiPlan ;

typedef struct EstructuraElemento  // REQUISITO O PROPOSITO USADO POR EL AGENTE
{
    char nombre[MAX_PALABRA];
    struct EstructuraElemento *sig ;
} TipoElemento ;

typedef struct EstructuraMensaje  // REQUISITO O PROPOSITO USADO POR EL AGENTE
{
    char variable[MAX_PALABRA];
    TipoAgente *enviador ;
    char contenido[250] ;
    int tipo ; // 1=dato, 2=agente esperando
    TipoHebra *quien ; // apunta a la hebra que esta esperando
    struct EstructuraMensaje *sig ;
} TipoMensaje ;


// fin del archivo